@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -209,7 +209,7 @@ enforces a specific mode. These are options that also exist to standard
 Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
 give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
-(do not use docstrings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
+(do not use doc strings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
 )
 
 parser.add_option(
@@ -389,7 +389,7 @@ times. Default empty.""",
 parser.add_option_group(follow_group)
 
 
-data_group = OptionGroup(parser, "Data files for standalone/onefile mode")
+data_group = OptionGroup(parser, "Data files")
 
 data_group.add_option(
     "--include-package-data",
@@ -405,18 +405,18 @@ empty.""",
 )
 
 data_group.add_option(
-    "--include-data-file",
+    "--include-data-files",
     action="append",
     dest="data_files",
     metavar="DESC",
     default=[],
     help="""\
 Include data files by filenames in the distribution. There are many
-allowed forms. With '--include-data-file=/path/to/file/*.txt=folder_name/some.txt' it
+allowed forms. With '--include-data-files=/path/to/file/*.txt=folder_name/some.txt' it
 will copy a single file and complain if it's multiple. With
-'--include-data-file=/path/to/files/*.txt=folder_name/' it will put
+'--include-data-files=/path/to/files/*.txt=folder_name/' it will put
 all matching files into that folder. For recursive copy there is a
-form with 3 values that '--include-data-file=/path/to/scan=folder_name=**/*.txt'
+form with 3 values that '--include-data-files=/path/to/scan=folder_name=**/*.txt'
 that will preserve directory structure. Default empty.""",
 )
 
@@ -428,31 +428,25 @@ data_group.add_option(
     default=[],
     help="""\
 Include data files from complete directory in the distribution. This is
-recursive. Check '--include-data-file' with patterns if you want non-recursive
-inclusion. An example would be '--include-data-dir=/path/somedir=data/somedir'
+recursive. Check '--include-data-files' with patterns if you want non-recursive
+inclusion. An example would be '--include-data-dir=/path/some_dir=data/some_dir'
 for plain copy, of the whole directory. All files are copied, if you want to
-exclude files you need to remove them beforehand. Default empty.""",
+exclude files you need to remove them beforehand, or use '--noinclude-data-files'
+option to remove them. Default empty.""",
 )
 
-data_files_tags = [("inhibit", "do not include the file")]
-
-# TODO: Expose this when finished, pylint: disable=using-constant-test
-if False:
-    data_group.add_option(
-        "--data-file-tags",
-        action="append",
-        dest="data_tags",
-        metavar="DATA_TAGS",
-        default=[],
-        help="""\
-    For included data files, special handlings can be chosen. With the
-    commercial plugins, e.g. files can be included directly in the
-    binary. The list is completed by some plugins. With the current
-    list of plugins, these are available: %s.
-    The default is empty."""
-        % ",".join("'%s' (%s)" % d for d in data_files_tags),
-    )
-
+data_group.add_option(
+    "--noinclude-data-files",
+    action="append",
+    dest="data_files_inhibited",
+    metavar="PATTERN",
+    default=[],
+    help="""\
+Do not include data files matching the filename pattern given. This is against
+the target filename, not source paths. So ignore file pattern from package
+data for "package_name" should be matched as "package_name/*.txt". Default
+empty.""",
+)
 
 parser.add_option_group(data_group)
 
@@ -487,9 +481,9 @@ execute_group.add_option(
     dest="keep_pythonpath",
     default=False,
     help="""\
-When immediately executing the created binary (--execute), don't reset
-PYTHONPATH. When all modules are successfully included, you ought to not need
-PYTHONPATH anymore.""",
+When immediately executing the created binary ('--execute'), don't reset
+'PYTHONPATH' environment. When all modules are successfully included, you
+ought to not need PYTHONPATH anymore.""",
 )
 
 parser.add_option_group(execute_group)
@@ -508,9 +502,22 @@ dump_group.add_option(
 parser.add_option_group(dump_group)
 
 
-codegen_group = OptionGroup(parser, "Code generation choices")
+compilation_group = OptionGroup(parser, "Compilation choices")
 
-codegen_group.add_option(
+compilation_group.add_option(
+    "--user-package-configuration-file",
+    action="append",
+    dest="user_yaml_files",
+    default=[],
+    metavar="USER_YAML",
+    help="""\
+User provided Yaml file with package configuration. You can include DLLs,
+remove bloat, add hidden dependencies. Check User Manual for a complete
+description of the format to use. Can be given multiple times. Defaults
+to empty.""",
+)
+
+compilation_group.add_option(
     "--disable-bytecode-cache",
     action="store_true",
     dest="disable_bytecode_cache",
@@ -520,7 +527,7 @@ Do not reuse dependency analysis results for modules, esp. from standard library
 that are included as bytecode.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--full-compat",
     action="store_false",
     dest="improved",
@@ -529,11 +536,11 @@ codegen_group.add_option(
 Enforce absolute compatibility with CPython. Do not even allow minor
 deviations from CPython behavior, e.g. not having better tracebacks
 or exception messages which are not really incompatible, but only
-different. This is intended for tests only and should not be used
-for normal use.""",
+different or worse. This is intended for tests only and should *not*
+be used.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--file-reference-choice",
     action="store",
     dest="file_reference_mode",
@@ -552,7 +559,23 @@ compatibility reasons, the "__file__" value will always have ".py" suffix
 independent of what it really is.""",
 )
 
-parser.add_option_group(codegen_group)
+compilation_group.add_option(
+    "--module-name-choice",
+    action="store",
+    dest="module_name_mode",
+    metavar="MODE",
+    choices=("original", "runtime"),
+    default=None,
+    help="""\
+Select what value "__name__" and "__package__" are going to be. With "runtime"
+(default for module mode), the created module uses the parent package to
+deduce the value of "__package__", to be fully compatible. The value "original"
+(default for other modes) allows for more static optimization to happen, but
+is incompatible for modules that normally can be loaded into any package.""",
+)
+
+
+parser.add_option_group(compilation_group)
 
 output_group = OptionGroup(parser, "Output choices")
 
@@ -622,9 +645,9 @@ production. Defaults to off.""",
 )
 
 debug_group.add_option(
-    "--unstripped",
+    "--unstriped",
     action="store_true",
-    dest="unstripped",
+    dest="unstriped",
     default=False,
     help="""\
 Keep debug info in the resulting object file for better debugger interaction.
@@ -643,7 +666,7 @@ Enable vmprof based profiling of time spent. Not working currently. Defaults to
 debug_group.add_option(
     "--internal-graph",
     action="store_true",
-    dest="graph",
+    dest="internal_graph",
     default=False,
     help="""\
 Create graph of optimization process internals, do not use for whole programs, but only
@@ -935,8 +958,7 @@ tracing_group.add_option(
     action="store_false",
     dest="progress_bar",
     default=True,
-    help="""Disable progress bar outputs (if tqdm is installed).
-Defaults to off.""",
+    help="""Disable progress bar. Defaults to off.""",
 )
 
 
@@ -1001,6 +1023,60 @@ Where to output --verbose, should be a filename. Default is standard output.""",
 
 parser.add_option_group(tracing_group)
 
+os_group = OptionGroup(parser, "General OS controls")
+
+os_group.add_option(
+    "--disable-console",
+    "--macos-disable-console",
+    "--windows-disable-console",
+    action="store_true",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, disable the console window and create a GUI
+application. Defaults to off.""",
+)
+
+os_group.add_option(
+    "--enable-console",
+    action="store_false",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, enable the console window and create a console
+application. This disables hints from certain modules, e.g. "PySide" that suggest
+to disable it. Defaults to true.""",
+)
+
+os_group.add_option(
+    "--force-stdout-spec",
+    "--windows-force-stdout-spec",
+    action="store",
+    dest="force_stdout_spec",
+    metavar="FORCE_STDOUT_SPEC",
+    default=None,
+    help="""\
+Force standard output of the program to go to this location. Useful for programs with
+disabled console and programs using the Windows Services Plugin of Nuitka commercial.
+Defaults to not active, use e.g. '%PROGRAM%.out.txt', i.e. file near your program.""",
+)
+
+os_group.add_option(
+    "--force-stderr-spec",
+    "--windows-force-stderr-spec",
+    action="store",
+    dest="force_stderr_spec",
+    metavar="FORCE_STDERR_SPEC",
+    default=None,
+    help="""\
+Force standard error of the program to go to this location. Useful for programs with
+disabled console and programs using the Windows Services Plugin of Nuitka commercial.
+Defaults to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your program.""",
+)
+
+
+parser.add_option_group(os_group)
+
 windows_group = OptionGroup(parser, "Windows specific controls")
 
 windows_group.add_option(
@@ -1011,15 +1087,6 @@ windows_group.add_option(
     help=SUPPRESS_HELP,
 )
 
-windows_group.add_option(
-    "--windows-disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
-    help="""\
-When compiling for Windows, disable the console window. Defaults to off.""",
-)
-
 windows_group.add_option(
     "--windows-icon-from-ico",
     action="append",
@@ -1061,7 +1128,7 @@ windows_group.add_option(
 )
 
 windows_group.add_option(
-    "--windows-uac-uiaccess",
+    "--windows-uac-uiaccess",  # spell-checker: ignore uiaccess
     action="store_true",
     dest="windows_uac_uiaccess",
     metavar="WINDOWS_UAC_UIACCESS",
@@ -1156,31 +1223,6 @@ windows_group.add_option(
 Use this as a temporary folder. Defaults to '%TEMP%\\onefile_%PID%_%TIME%', i.e. system temporary directory.""",
 )
 
-windows_group.add_option(
-    "--windows-force-stdout-spec",
-    action="store",
-    dest="force_stdout_spec",
-    metavar="WINDOWS_FORCE_STDOUT_SPEC",
-    default=None,
-    help="""\
-Force standard output of the program to go to this location. Useful for programs with
-disabled console and programs using the Windows Services Plugin of Nuitka. Defaults
-to not active, use e.g. '%PROGRAM%.out.txt', i.e. file near your program.""",
-)
-
-windows_group.add_option(
-    "--windows-force-stderr-spec",
-    action="store",
-    dest="force_stderr_spec",
-    metavar="WINDOWS_FORCE_STDERR_SPEC",
-    default=None,
-    help="""\
-Force standard error of the program to go to this location. Useful for programs with
-disabled console and programs using the Windows Services Plugin of Nuitka. Defaults
-to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your program.""",
-)
-
-
 parser.add_option_group(windows_group)
 
 macos_group = OptionGroup(parser, "macOS specific controls")
@@ -1198,17 +1240,6 @@ is what the running Python allows for. Default is "native" which is
 the architecture the Python is run with.""",
 )
 
-macos_group.add_option(
-    "--macos-disable-console",
-    "--disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
-    help="""\
-When compiling for macOS, disable the console window and create a GUI
-application. Defaults to off.""",
-)
-
 macos_group.add_option(
     "--macos-create-app-bundle",
     action="store_true",
@@ -1221,12 +1252,12 @@ is the only way to unlock disabling of console.Defaults to off.""",
 )
 
 macos_group.add_option(
-    "--macos-onefile-icon",
+    "--macos-app-icon",
     action="append",
     dest="icon_path",
     metavar="ICON_PATH",
     default=[],
-    help="Add executable icon for binary to use. Can be given only one time. Defaults to Python icon if available.",
+    help="Add icon for the application bundle to use. Can be given only one time. Defaults to Python icon if available.",
 )
 
 
@@ -1237,9 +1268,9 @@ macos_group.add_option(
     metavar="MACOS_SIGNED_APP_NAME",
     default=None,
     help="""\
-Name of the application to use for macOS signing. Follow com.yourcompany.appname naming
-results for best results, as these have to be globally unique, and will grant protected
-API accesses.""",
+Name of the application to use for macOS signing. Follow "com.YourCompany.AppName"
+naming results for best results, as these have to be globally unique, and will
+potentially grant protected API accesses.""",
 )
 
 macos_group.add_option(
@@ -1253,6 +1284,19 @@ Name of the product to use in macOS bundle information. Defaults to base
 filename of the binary.""",
 )
 
+macos_group.add_option(
+    "--macos-sign-identity",
+    action="store",
+    dest="macos_sign_identity",
+    metavar="MACOS_APP_VERSION",
+    default="-",
+    help="""\
+When signing on macOS, by default an ad-hoc identify will be used, but with this
+option your get to specify another identity to use. The signing of code is now
+mandatory on macOS and cannot be disabled. Default "-" if not given, which means
+ad-hoc.""",
+)
+
 macos_group.add_option(
     "--macos-app-version",
     action="store",
@@ -1260,10 +1304,26 @@ macos_group.add_option(
     metavar="MACOS_APP_VERSION",
     default=None,
     help="""\
-Product version to use in macOS bundle information. Defaults to 1.0 if
+Product version to use in macOS bundle information. Defaults to "1.0" if
 not given.""",
 )
 
+macos_group.add_option(
+    "--macos-app-protected-resource",
+    action="append",
+    dest="macos_protected_resources",
+    metavar="RESOURCE_DESC",
+    default=[],
+    help="""\
+Request access for macOS protected resources, e.g.
+"NSMicrophoneUsageDescription:Microphone access for recording audio."
+requests access to the microphone and provides an informative text for
+the user, why that is needed. Before the colon, is an OS identifier for
+an access right, then the informative text. Legal values can be found on
+https://developer.apple.com/documentation/bundleresources/information_property_list/protected_resources and
+the option can be specified multiple times. Default empty.""",
+)
+
 
 parser.add_option_group(macos_group)
 
@@ -1281,7 +1341,7 @@ linux_group.add_option(
 linux_group.add_option(
     "--linux-onefile-compression",
     action="store",
-    dest="appimage_compression",
+    dest="app_image_compression",
     choices=("gzip", "xz"),
     metavar="COMPRESSION",
     default="gzip",
@@ -1351,15 +1411,13 @@ plugin_group.add_option(
 )
 
 plugin_group.add_option(
-    "--persist-source-changes",
+    "--show-source-changes",
     action="store_true",
-    dest="persist_source_changes",
+    dest="show_source_changes",
     default=False,
     help="""\
-Write source changes to original Python files. Use with care. May need
-permissions, best for use in a virtualenv to debug if plugin code
-changes work with standard Python or to benefit from bloat removal
-even with pure Python. Default False.""",
+Show source changes to original Python file content before compilation. Mostly
+intended for developing plugins. Default False.""",
 )
 
 
@@ -1367,11 +1425,11 @@ def _considerPluginOptions(logger):
     # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
-        addStandardPluginCommandlineOptions,
+        addStandardPluginCommandLineOptions,
         addUserPluginCommandLineOptions,
     )
 
-    addStandardPluginCommandlineOptions(parser=parser, data_files_tags=data_files_tags)
+    addStandardPluginCommandLineOptions(parser=parser)
 
     for arg in sys.argv[1:]:
         if arg.startswith(("--enable-plugin=", "--plugin-enable=")):
@@ -1385,7 +1443,6 @@ def _considerPluginOptions(logger):
             addPluginCommandLineOptions(
                 parser=parser,
                 plugin_names=plugin_names.split(","),
-                data_files_tags=data_files_tags,
             )
 
         if arg.startswith("--user-plugin="):
@@ -1396,9 +1453,7 @@ def _considerPluginOptions(logger):
                     % plugin_name.split("=", 1)[0]
                 )
 
-            addUserPluginCommandLineOptions(
-                parser=parser, filename=plugin_name, data_files_tags=data_files_tags
-            )
+            addUserPluginCommandLineOptions(parser=parser, filename=plugin_name)
 
 
 def _expandProjectArg(arg, filename_arg, for_eval):
@@ -1435,7 +1490,7 @@ def _expandProjectArg(arg, filename_arg, for_eval):
 
 
 def _getProjectOptions(logger, filename_arg, module_mode):
-    # Complex stuff, pylint: disable=too-many-branches,too-many-locals
+    # Complex stuff, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     if os.path.isdir(filename_arg):
         if module_mode:
@@ -1457,8 +1512,8 @@ def _getProjectOptions(logger, filename_arg, module_mode):
 
     cond_level = -1
 
-    for count, line in enumerate(contents_by_line):
-        match = re.match(b"^\\s*#(\\s+)nuitka-project(.*?):(.*)", line)
+    for line_number, line in enumerate(contents_by_line):
+        match = re.match(b"^\\s*#(\\s*)nuitka-project(.*?):(.*)", line)
 
         if match:
             level, command, arg = match.groups()
@@ -1468,13 +1523,15 @@ def _getProjectOptions(logger, filename_arg, module_mode):
             # Check for empty conditional blocks.
             if expect_block and level <= cond_level:
                 sysexit(
-                    count,
-                    "Error, 'nuitka-project-if' is expected to be followed by block start.",
+                    line_number,
+                    "Error, 'nuitka-project-if|else' is expected to be followed by block start.",
                 )
 
             expect_block = False
 
-            if level <= cond_level:
+            if level == cond_level and command == b"-else":
+                execute_block = not execute_block
+            elif level <= cond_level:
                 execute_block = True
 
             if level > cond_level and not execute_block:
@@ -1487,11 +1544,11 @@ def _getProjectOptions(logger, filename_arg, module_mode):
             if command == "-if":
                 if not arg.endswith(":"):
                     sysexit(
-                        count,
+                        line_number,
                         "Error, 'nuitka-project-if' needs to start a block with a colon at line end.",
                     )
 
-                arg = arg[:-1]
+                arg = arg[:-1].strip()
 
                 expanded = _expandProjectArg(arg, filename_arg, for_eval=True)
 
@@ -1503,11 +1560,26 @@ def _getProjectOptions(logger, filename_arg, module_mode):
                 # Likely mistakes, e.g. with "in" tests.
                 if r is not True and r is not False:
                     sys.exit(
-                        "Error, 'nuitka-project-if' condition %r (%r) does not yield boolean result %r"
+                        "Error, 'nuitka-project-if' condition %r (expanded to %r) does not yield boolean result %r"
                         % (arg, expanded, r)
                     )
 
                 execute_block = r
+                expect_block = True
+                cond_level = level
+            elif command == "-else":
+                if arg:
+                    sysexit(
+                        line_number,
+                        "Error, 'nuitka-project-else' cannot have argument.",
+                    )
+
+                if cond_level != level:
+                    sysexit(
+                        line_number,
+                        "Error, 'nuitka-project-else' not currently allowed after nested nuitka-project-if.",
+                    )
+
                 expect_block = True
                 cond_level = level
             elif command == "":
