@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,6 +23,8 @@ from nuitka import Options
 from nuitka.__past__ import long
 from nuitka.PythonVersions import python_version
 
+# spell-checker: ignore fromlist
+
 
 def getConstantDefaultPopulation():
     """These are values for non-trivial constants.
@@ -94,6 +96,7 @@ def getConstantDefaultPopulation():
         "level",
         # Meta path based loader.
         "read",
+        "rb",
     ]
 
     # Pickling of instance methods.
