@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -313,7 +313,7 @@ PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaP
 
         _initSlotCompare();
 #if PYTHON_VERSION >= 0x270
-        _initSlotIternext();
+        _initSlotIterNext();
 #endif
 
         patchTypeComparison();
@@ -523,11 +523,13 @@ static PyObject *orig_dunder_file_value;
 static int Nuitka_TopLevelModule_tp_setattro(PyObject *module, PyObject *name, PyObject *value) {
     PyModule_Type.tp_setattro = orig_PyModule_Type_tp_setattro;
 
-    UPDATE_STRING_DICT0(
-        moduledict_%(module_identifier)s,
-        (Nuitka_StringObject *)const_str_plain___file__,
-        orig_dunder_file_value
-    );
+    if (orig_dunder_file_value != NULL) {
+        UPDATE_STRING_DICT0(
+            moduledict_%(module_identifier)s,
+            (Nuitka_StringObject *)const_str_plain___file__,
+            orig_dunder_file_value
+        );
+    }
 
     // Prevent "__spec__" update as well.
 #if PYTHON_VERSION >= 0x340
