@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,13 +31,12 @@ import inspect
 import os
 from optparse import OptionConflictError, OptionGroup
 
-import nuitka.plugins.commercial
-import nuitka.plugins.standard
 from nuitka import Options, OutputDirectories
 from nuitka.__past__ import basestring, iter_modules, iterItems
 from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
+from nuitka.freezer.IncludedDataFiles import IncludedDataFile
 from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
@@ -61,7 +60,6 @@ from .PluginBase import (
 active_plugins = OrderedDict()
 plugin_name2plugin_classes = {}
 plugin_options = {}
-plugin_datatag2pluginclasses = {}
 plugin_values = {}
 user_plugins = OrderedSet()
 
@@ -188,9 +186,9 @@ def _loadPluginClassesFromPackage(scan_package):
         try:
             plugin_module = module_loader.load_module(item.name)
         except Exception:
-            if Options.is_nondebug:
+            if Options.is_non_debug:
                 plugins_logger.warning(
-                    "Problem loading plugin %r (%s), ignored. Use --debug to make it visible."
+                    "Problem loading plugin %r (%s), ignored. Use '--debug' to make it visible."
                     % (item.name, module_loader.get_filename())
                 )
                 continue
@@ -251,14 +249,21 @@ def loadStandardPluginClasses():
     """Load plugin files located in 'standard' folder.
 
     Notes:
-        Scan through the 'standard' and 'commercial' sub-folder of the folder
-        where this module resides. Import each valid Python module (but not
-        packages) and process it as a plugin.
+        Scan through the 'standard' and 'commercial' plugins. Import each valid
+        Python module (but not packages) and process it as a plugin.
     Returns:
         None
     """
+    import nuitka.plugins.standard
+
     _loadPluginClassesFromPackage(nuitka.plugins.standard)
-    _loadPluginClassesFromPackage(nuitka.plugins.commercial)
+
+    try:
+        import nuitka.plugins.commercial
+    except ImportError:
+        pass
+    else:
+        _loadPluginClassesFromPackage(nuitka.plugins.commercial)
 
 
 class Plugins(object):
@@ -292,7 +297,13 @@ class Plugins(object):
 
                 yield ModuleName(v)
 
+        seen = set()
+
         for full_name in iterateModuleNames(plugin.getImplicitImports(module)):
+            if full_name in seen:
+                continue
+            seen.add(full_name)
+
             try:
                 _module_name, module_filename, _finding = Importing.locateModule(
                     module_name=full_name,
@@ -416,15 +427,21 @@ class Plugins(object):
                     os.path.join(dist_dir, os.path.basename(dll_filename))
                 )
 
+    @staticmethod
+    def onBeforeCodeParsing():
+        """Let plugins prepare for code parsing"""
+        for plugin in getActivePlugins():
+            plugin.onBeforeCodeParsing()
+
     @staticmethod
     def onStandaloneDistributionFinished(dist_dir):
-        """Let plugins postprocess the distribution folder in standalone mode"""
+        """Let plugins post-process the distribution folder in standalone mode"""
         for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(dist_dir)
 
     @staticmethod
     def onOnefileFinished(filename):
-        """Let plugins postprocess the onefile executable in onefile mode"""
+        """Let plugins post-process the onefile executable in onefile mode"""
         for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(filename)
 
@@ -543,10 +560,39 @@ class Plugins(object):
             Data file description pairs, either (source, dest) or (func, dest)
             where the func will be called to create the content dynamically.
         """
+
+        def _iterateIncludedDataFiles(plugin, value):
+            if value is None:
+                pass
+            elif isinstance(value, IncludedDataFile):
+                yield value
+            elif inspect.isgenerator(value):
+                for val in value:
+                    for v in _iterateIncludedDataFiles(plugin, val):
+                        yield v
+            else:
+                plugin.sysexit("Plugin return non-datafile '%s'" % repr(value))
+
         for plugin in getActivePlugins():
             for value in plugin.considerDataFiles(module):
-                if value:
-                    yield plugin, value
+                for included_datafile in _iterateIncludedDataFiles(plugin, value):
+                    yield included_datafile
+
+    @staticmethod
+    def getDataFileTags(included_datafile):
+        tags = OrderedSet([included_datafile.kind])
+
+        tags.update(Options.getDataFileTags(tags))
+
+        for plugin in getActivePlugins():
+            plugin.updateDataFileTags(included_datafile)
+
+        return tags
+
+    @staticmethod
+    def onDataFileTags(included_datafile):
+        for plugin in getActivePlugins():
+            plugin.onDataFileTags(included_datafile)
 
     @classmethod
     def _createTriggerLoadedModule(cls, module, trigger_name, code, flags):
@@ -617,7 +663,9 @@ class Plugins(object):
                     descs = [descs]
 
                 for desc in descs:
-                    if len(desc) == 2:
+                    if desc is None:
+                        pass
+                    elif len(desc) == 2:
                         code, reason = desc
                         flags = ()
                     else:
@@ -773,12 +821,15 @@ class Plugins(object):
         return bytecode
 
     @staticmethod
-    def onModuleEncounter(module_filename, module_name, module_kind):
+    def onModuleEncounter(module_name, module_filename, module_kind):
         result = None
+        deciding_plugins = []
 
         for plugin in getActivePlugins():
             must_recurse = plugin.onModuleEncounter(
-                module_filename, module_name, module_kind
+                module_name=module_name,
+                module_filename=module_filename,
+                module_kind=module_kind,
             )
 
             if must_recurse is None:
@@ -791,12 +842,26 @@ class Plugins(object):
 
             if result is not None:
                 # false alarm, pylint: disable=unsubscriptable-object
-                assert result[0] == must_recurse[0]
+                if result[0] != must_recurse[0]:
+                    plugin.sysexit(
+                        "Error, decision %s does not match other plugin '%s' decision."
+                        % (must_recurse[0], ".".join(deciding_plugins))
+                    )
 
+            deciding_plugins.append(plugin.plugin_name)
             result = must_recurse
 
         return result
 
+    @staticmethod
+    def onModuleRecursion(module_name, module_filename, module_kind):
+        for plugin in getActivePlugins():
+            plugin.onModuleRecursion(
+                module_name=module_name,
+                module_filename=module_filename,
+                module_kind=module_kind,
+            )
+
     @staticmethod
     def onModuleInitialSet():
         """The initial set of root modules is complete, plugins may add more."""
@@ -973,6 +1038,11 @@ class Plugins(object):
 
         return cls.extra_link_directories
 
+    @classmethod
+    def onDataComposerRun(cls):
+        for plugin in getActivePlugins():
+            plugin.onDataComposerRun()
+
     @classmethod
     def onDataComposerResult(cls, blob_filename):
         for plugin in getActivePlugins():
@@ -1010,6 +1080,12 @@ class Plugins(object):
                 body=body,
             )
 
+    @classmethod
+    def getCacheContributionValues(cls, module_name):
+        for plugin in getActivePlugins():
+            for value in plugin.getCacheContributionValues(module_name):
+                yield value
+
 
 def listPlugins():
     """Print available standard plugins."""
@@ -1108,7 +1184,7 @@ def loadPlugins():
         loadStandardPluginClasses()
 
 
-def addStandardPluginCommandlineOptions(parser, data_files_tags):
+def addStandardPluginCommandLineOptions(parser):
     loadPlugins()
 
     for (_plugin_name, (plugin_class, _plugin_detector)) in sorted(
@@ -1118,7 +1194,6 @@ def addStandardPluginCommandlineOptions(parser, data_files_tags):
             _addPluginCommandLineOptions(
                 parser=parser,
                 plugin_class=plugin_class,
-                data_files_tags=data_files_tags,
             )
 
 
@@ -1193,7 +1268,7 @@ def lateActivatePlugin(plugin_name, option_values):
     _addActivePlugin(getPluginClass(plugin_name), args=True, force=True)
 
 
-def _addPluginCommandLineOptions(parser, plugin_class, data_files_tags):
+def _addPluginCommandLineOptions(parser, plugin_class):
     plugin_name = plugin_class.plugin_name
 
     if plugin_name not in plugin_options:
@@ -1219,26 +1294,12 @@ def _addPluginCommandLineOptions(parser, plugin_class, data_files_tags):
         else:
             plugin_options[plugin_name] = ()
 
-        plugin_data_files_tags = plugin_class.getTagDataFileTagOptions()
 
-        if plugin_data_files_tags:
-            for tag_name, tag_desc in plugin_data_files_tags:
-                if tag_name in (tag for tag, _desc in data_files_tags):
-                    plugins_logger.sysexit(
-                        "Plugin '%s' provides data files tag handling '%s' already provided."
-                        % (plugin_name, tag_name)
-                    )
-
-                data_files_tags.append((tag_name, tag_desc))
-
-                plugin_datatag2pluginclasses[tag_name] = plugin_class
-
-
-def addPluginCommandLineOptions(parser, plugin_names, data_files_tags):
+def addPluginCommandLineOptions(parser, plugin_names):
     """Add option group for the plugin to the parser.
 
     Notes:
-        This is exclusively for use in the commandline parsing. Not all
+        This is exclusively for use in the command line parsing. Not all
         plugins have to have options. But this will add them to the
         parser in a first pass, so they can be recognized in a second
         pass with them included.
@@ -1248,16 +1309,12 @@ def addPluginCommandLineOptions(parser, plugin_names, data_files_tags):
     """
     for plugin_name in plugin_names:
         plugin_class = getPluginClass(plugin_name)
-        _addPluginCommandLineOptions(
-            parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
-        )
+        _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
 
 
-def addUserPluginCommandLineOptions(parser, filename, data_files_tags):
+def addUserPluginCommandLineOptions(parser, filename):
     plugin_class = loadUserPlugin(filename)
-    _addPluginCommandLineOptions(
-        parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
-    )
+    _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
 
     user_plugins.add(plugin_class)
 
