@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,7 @@
 
 import nuitka.Options
 
-nuitka.Options.is_fullcompat = False
+nuitka.Options.is_full_compat = False
 
 # isort:start
 
@@ -32,6 +32,7 @@ from abc import abstractmethod
 import nuitka.codegen.ComparisonCodes
 import nuitka.codegen.HelperDefinitions
 import nuitka.codegen.Namify
+import nuitka.specs.BuiltinBytesOperationSpecs
 import nuitka.specs.BuiltinDictOperationSpecs
 import nuitka.specs.BuiltinStrOperationSpecs
 import nuitka.specs.BuiltinUnicodeOperationSpecs
@@ -39,12 +40,17 @@ from nuitka.__past__ import getMetaClassBase, long
 from nuitka.codegen.CallCodes import (
     getQuickCallCode,
     getQuickMethodCallCode,
-    getQuickMethodDescrCallCode,
+    getQuickMethodDescriptorCallCode,
     getQuickMixedCallCode,
     getTemplateCodeDeclaredFunction,
     max_quick_call,
 )
-from nuitka.nodes.ImportNodes import hard_modules
+from nuitka.codegen.ImportCodes import getImportModuleHardCodeName
+from nuitka.nodes.ImportNodes import (
+    hard_modules,
+    hard_modules_non_stdlib,
+    hard_modules_version,
+)
 from nuitka.utils.Jinja2 import getTemplateC
 
 from .Common import (
@@ -53,10 +59,11 @@ from .Common import (
     python2_dict_methods,
     python2_str_methods,
     python2_unicode_methods,
+    python3_bytes_methods,
     python3_dict_methods,
     python3_str_methods,
-    withFileOpenedAndAutoformatted,
-    writeline,
+    withFileOpenedAndAutoFormatted,
+    writeLine,
 )
 
 
@@ -1834,14 +1841,14 @@ def makeHelpersComparisonOperation(operand, op_code):
     filename_c = "nuitka/build/static_src/HelpersComparison%s.c" % op_code.capitalize()
     filename_h = "nuitka/build/include/nuitka/helper/comparisons_%s.h" % op_code.lower()
 
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -1881,14 +1888,14 @@ def makeHelpersBinaryOperation(operand, op_code):
         "nuitka/build/include/nuitka/helper/operations_binary_%s.h" % op_code.lower()
     )
 
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -1930,14 +1937,14 @@ def makeHelpersInplaceOperation(operand, op_code):
         "nuitka/build/include/nuitka/helper/operations_inplace_%s.h" % op_code.lower()
     )
 
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -1970,14 +1977,14 @@ def makeHelpersImportHard():
 
     template = getDoExtensionUsingTemplateC("HelperImportHard.c.j2")
 
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -1999,12 +2006,21 @@ def makeHelpersImportHard():
 
 def makeHelperImportModuleHard(template, module_name, emit_h, emit_c, emit):
     emit('/* C helper for hard import of module "%s" import. */' % module_name)
-    emit()
 
-    if module_name == "_frozen_importlib":
-        python_requirement = "PYTHON_VERSION >= 0x300"
-    elif module_name == "_frozen_importlib_external":
-        python_requirement = "PYTHON_VERSION >= 0x350"
+    python_min_max_version = hard_modules_version.get(module_name)
+
+    if python_min_max_version is not None:
+        python_min_version, python_max_version = python_min_max_version
+
+        parts = []
+
+        if python_min_version is not None:
+            parts.append("PYTHON_VERSION >= %s" % hex(python_min_version))
+        if python_max_version is not None:
+            parts.append("PYTHON_VERSION < %s" % hex(python_max_version))
+
+        python_requirement = " && ".join(parts)
+
     else:
         python_requirement = None
 
@@ -2012,7 +2028,11 @@ def makeHelperImportModuleHard(template, module_name, emit_h, emit_c, emit):
         emit("#if %s" % python_requirement)
 
     code = template.render(
-        module_name=module_name, name=template.name, target=object_desc
+        module_name=module_name,
+        module_code_name=getImportModuleHardCodeName(module_name),
+        name=template.name,
+        target=object_desc,
+        is_stdlib=module_name not in hard_modules_non_stdlib,
     )
 
     emit_c(code)
@@ -2021,20 +2041,22 @@ def makeHelperImportModuleHard(template, module_name, emit_h, emit_c, emit):
     if python_requirement:
         emit("#endif")
 
+    emit()
+
 
 def makeHelperCalls():
     filename_c = "nuitka/build/static_src/HelpersCalling2.c"
     filename_h = "nuitka/build/include/nuitka/helper/calling2.h"
 
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
                 assert args[0] != "extern "
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -2089,7 +2111,7 @@ def makeHelperCalls():
                         emit_h(getTemplateCodeDeclaredFunction(code))
 
             for args_count in range(1, 5):
-                code = getQuickMethodDescrCallCode(args_count=args_count)
+                code = getQuickMethodDescriptorCallCode(args_count=args_count)
 
                 emit_c(code)
                 emit_h(getTemplateCodeDeclaredFunction(code))
@@ -2102,39 +2124,62 @@ def makeHelperCalls():
 
 
 def _makeHelperBuiltinTypeAttributes(
-    type_prefix,
-    type_name,
-    python2_methods,
-    python3_methods,
-    emit_c,
+    type_prefix, type_name, python2_methods, python3_methods, emit_c, emit_h
 ):
+    # many cases to deal with, pylint: disable=too-many-branches
+
     def getVarName(method_name):
         return "%s_builtin_%s" % (type_prefix, method_name)
 
     for method_name in sorted(set(python2_methods + python3_methods)):
+        is_public = method_name in ("format",)
+
         if method_name in python2_methods and method_name not in python3_methods:
             emit_c("#if PYTHON_VERSION < 0x300")
+            if is_public:
+                emit_h("#if PYTHON_VERSION < 0x300")
             needs_endif = True
         elif method_name not in python2_methods and method_name in python3_methods:
             emit_c("#if PYTHON_VERSION >= 0x300")
+            if is_public:
+                emit_h("#if PYTHON_VERSION >= 0x300")
             needs_endif = True
         else:
             needs_endif = False
 
-        emit_c("static PyObject *%s = NULL;" % getVarName(method_name))
+        if not is_public:
+            emit_c("static")
+
+        emit_c("PyObject *%s = NULL;" % getVarName(method_name))
+
+        if is_public:
+            emit_h("extern PyObject *%s;" % getVarName(method_name))
 
         if needs_endif:
             emit_c("#endif")
 
+            if is_public:
+                emit_h("#endif")
+
     if not python3_methods:
         emit_c("#if PYTHON_VERSION < 0x300")
+    if not python2_methods:
+        emit_c("#if PYTHON_VERSION >= 0x300")
 
     emit_c("static void _init%sBuiltinMethods(void) {" % type_prefix.capitalize())
     for method_name in sorted(set(python2_methods + python3_methods)):
-        if method_name in python2_methods and method_name not in python3_methods:
+        if (
+            method_name in python2_methods
+            and method_name not in python3_methods
+            and python3_methods
+        ):
             emit_c("#if PYTHON_VERSION < 0x300")
             needs_endif = True
-        elif method_name not in python2_methods and method_name in python3_methods:
+        elif (
+            method_name not in python2_methods
+            and method_name in python3_methods
+            and python2_methods
+        ):
             emit_c("#if PYTHON_VERSION >= 0x300")
             needs_endif = True
         else:
@@ -2150,7 +2195,7 @@ def _makeHelperBuiltinTypeAttributes(
 
     emit_c("}")
 
-    if not python3_methods:
+    if not python2_methods or not python3_methods:
         emit_c("#endif")
 
 
@@ -2165,9 +2210,10 @@ generate_builtin_type_operations = [
         nuitka.specs.BuiltinDictOperationSpecs,
         ("pop", "setdefault"),
     ),
-    # TODO: These are very complex things using stringlib in Python, that we do not have easy access to,
-    # but we might one day for Nuitka-Python expose it for the static linking of it and then we
-    # could in fact call these directly.
+    # TODO: These are very complex things using "string lib" code in CPython,
+    # that we do not have easy access to, but we might one day for Nuitka-Python
+    # expose it for the static linking of it and then we could in fact call
+    # these directly.
     (
         "tshape_str",
         str_desc,
@@ -2201,6 +2247,7 @@ generate_builtin_type_operations = [
             "replace",
             "encode",
             "decode",
+            "count",
         ),
     ),
     # TODO: This is using Python2 spec module for Python3 strings, that will be a problem down the
@@ -2235,8 +2282,15 @@ generate_builtin_type_operations = [
             "endswith",
             "replace",
             "encode",
+            "count",
         ),
     ),
+    (
+        "tshape_bytes",
+        bytes_desc,
+        nuitka.specs.BuiltinBytesOperationSpecs,
+        ("decode",),
+    ),
 ]
 
 
@@ -2244,14 +2298,14 @@ def makeHelperBuiltinTypeMethods():
     # Many details, pylint: disable=too-many-locals
     filename_c = "nuitka/build/static_src/HelpersBuiltinTypeMethods.c"
     filename_h = "nuitka/build/include/nuitka/helper/operations_builtin_types.h"
-    with withFileOpenedAndAutoformatted(filename_c) as output_c:
-        with withFileOpenedAndAutoformatted(filename_h) as output_h:
+    with withFileOpenedAndAutoFormatted(filename_c) as output_c:
+        with withFileOpenedAndAutoFormatted(filename_h) as output_h:
 
             def emit_h(*args):
-                writeline(output_h, *args)
+                writeLine(output_h, *args)
 
             def emit_c(*args):
-                writeline(output_c, *args)
+                writeLine(output_c, *args)
 
             def emit(*args):
                 emit_h(*args)
@@ -2260,11 +2314,10 @@ def makeHelperBuiltinTypeMethods():
             emitIDE(emit)
 
             _makeHelperBuiltinTypeAttributes(
-                "str",
-                "PyString_Type",
-                python2_str_methods,
-                (),
-                emit_c,
+                "str", "PyString_Type", python2_str_methods, (), emit_c, emit_h
+            )
+            _makeHelperBuiltinTypeAttributes(
+                "bytes", "PyBytes_Type", (), python3_bytes_methods, emit_c, emit_h
             )
             _makeHelperBuiltinTypeAttributes(
                 "unicode",
@@ -2272,6 +2325,7 @@ def makeHelperBuiltinTypeMethods():
                 python2_unicode_methods,
                 python3_str_methods,
                 emit_c,
+                emit_h,
             )
             _makeHelperBuiltinTypeAttributes(
                 "dict",
@@ -2279,6 +2333,7 @@ def makeHelperBuiltinTypeMethods():
                 python2_dict_methods,
                 python3_dict_methods,
                 emit_c,
+                emit_h,
             )
 
             template = getDoExtensionUsingTemplateC("HelperBuiltinMethodOperation.c.j2")
@@ -2296,6 +2351,7 @@ def makeHelperBuiltinTypeMethods():
                     (
                         present,
                         arg_names,
+                        _arg_tests,
                         arg_name_mapping,
                         arg_counts,
                     ) = getMethodVariations(
