@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,21 +21,20 @@
 
 """
 
-from __future__ import print_function
-
 import hashlib
 import os
 import pickle
 import re
 import subprocess
 import sys
-import tempfile
 import time
 
 from nuitka.PythonVersions import python_version
 from nuitka.tools.testing.Common import (
     addToPythonPath,
     executeAfterTimePassed,
+    getDebugPython,
+    getTempDir,
     getTestingCPythonOutputsCacheDir,
     killProcess,
     withPythonPathChange,
@@ -43,7 +42,9 @@ from nuitka.tools.testing.Common import (
 from nuitka.tools.testing.OutputComparison import compareOutput
 from nuitka.Tracing import my_print
 from nuitka.utils.Execution import (
+    callProcess,
     check_output,
+    executeProcess,
     wrapCommandForDebuggerForSubprocess,
 )
 from nuitka.utils.Importing import getSharedLibrarySuffix
@@ -88,12 +89,13 @@ def checkNoPermissionError(output):
 def _getCPythonResults(cpython_cmd, send_kill):
     stop_watch = StopWatch()
 
-    # Try a coupile of times for permission denied, on Windows it can
+    # Try a compile of times for permission denied, on Windows it can
     # be transient.
     for _i in range(5):
         stop_watch.start()
 
         with withPythonPathChange(os.getcwd()):
+            # want fine grained control, pylint: disable=consider-using-with
             process = subprocess.Popen(
                 args=cpython_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
             )
@@ -200,6 +202,23 @@ def main():
         else:
             return False
 
+    def hasArgValue(arg_option, default=None):
+        for arg in args:
+            if arg.startswith(arg_option + "="):
+                args.remove(arg)
+                return arg[len(arg_option) + 1 :]
+        return default
+
+    def hasArgValues(arg_option):
+        result = []
+
+        for arg in tuple(args):
+            if arg.startswith(arg_option + "="):
+                args.remove(arg)
+                result.append(arg[len(arg_option) + 1 :])
+
+        return result
+
     # For output keep it
     arguments = list(args)
 
@@ -208,8 +227,10 @@ def main():
     ignore_warnings = hasArg("ignore_warnings")
     expect_success = hasArg("expect_success")
     expect_failure = hasArg("expect_failure")
-    python_debug = hasArg("python_debug")
-    module_mode = hasArg("module_mode")
+    python_debug = hasArg("python_debug") or hasArg("--python-debug")
+    module_mode = hasArg("--module")
+    module_entry_point = hasArgValue("--module-entry-point")
+    coverage_mode = hasArg("coverage")
     two_step_execution = hasArg("two_step_execution")
     binary_python_path = hasArg("binary_python_path")
     keep_python_path = hasArg("keep_python_path")
@@ -220,11 +241,11 @@ def main():
     remove_binary = not hasArg("--keep-binary")
     standalone_mode = hasArg("--standalone")
     onefile_mode = hasArg("--onefile")
-    no_site = hasArg("no_site")
+    no_site = hasArg("no_site") or coverage_mode
+    report = hasArgValue("--report")
     nofollow_imports = hasArg("recurse_none") or hasArg("--nofollow-imports")
     follow_imports = hasArg("recurse_all") or hasArg("--follow-imports")
     timing = hasArg("timing")
-    coverage_mode = hasArg("coverage")
     original_file = hasArg("original_file") or hasArg(
         "--file-reference-choice=original"
     )
@@ -237,6 +258,10 @@ def main():
     noverbose_log = hasArg("noverbose_log")
     noinclusion_log = hasArg("noinclusion_log")
     send_kill = hasArg("--send-ctrl-c")
+    output_dir = hasArgValue("--output-dir", None)
+    include_packages = hasArgValues("--include-package")
+    include_modules = hasArgValues("--include-module")
+    python_flag_m = hasArg("--python-flag=-m")
 
     plugins_enabled = []
     for count, arg in reversed(tuple(enumerate(args))):
@@ -302,22 +327,13 @@ def main():
 
     extra_options = os.environ.get("NUITKA_EXTRA_OPTIONS", "").split()
 
-    if "--python-debug" in extra_options or "--python-dbg" in extra_options:
+    if os.path.normcase(os.environ["PYTHON"]).endswith(("-dbg", "-debug", "_d.exe")):
+        python_debug = True
+    elif "--python-debug" in extra_options or "--python-dbg" in extra_options:
         python_debug = True
 
     if python_debug:
-        if os.path.exists(os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")):
-            os.environ["PYTHON"] += "-dbg"
-
-        if os.name == "nt":
-            if os.path.exists(os.environ["PYTHON"][:-4] + "_d.exe"):
-                os.environ["PYTHON"] = os.environ["PYTHON"][:-4] + "_d.exe"
-
-    if os.environ["PYTHON"].endswith("-dbg"):
-        python_debug = True
-
-    if os.environ["PYTHON"].lower().endswith("_d.exe"):
-        python_debug = True
+        os.environ["PYTHON"] = getDebugPython() or os.environ["PYTHON"]
 
     if comparison_mode:
         my_print(
@@ -347,28 +363,41 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         filename = os.path.abspath(filename)
 
     if module_mode:
+        module_name = os.path.basename(filename)
+
+        if module_name.endswith(".py"):
+            module_name = module_name[:-3]
+
+        mini_script = "import %s" % module_name
+        if module_entry_point:
+            mini_script += "; %s.%s()" % (module_name, module_entry_point)
+
+        cpython_cmd = [
+            os.environ["PYTHON"],
+            "-c",
+            "import sys; sys.path.append(%s); %s"
+            % (repr(os.path.dirname(filename)), mini_script),
+        ]
+
         if no_warnings:
-            cpython_cmd = [
-                os.environ["PYTHON"],
+            cpython_cmd[1:1] = [
                 "-W",
                 "ignore",
-                "-c",
-                "import sys; sys.path.append(%s); import %s"
-                % (repr(os.path.dirname(filename)), os.path.basename(filename)),
             ]
-        else:
-            cpython_cmd = [
-                os.environ["PYTHON"],
-                "-c",
-                "import sys; sys.path.append(%s); import %s"
-                % (repr(os.path.dirname(filename)), os.path.basename(filename)),
+    else:
+        cpython_cmd = [os.environ["PYTHON"]]
+
+        if no_warnings:
+            cpython_cmd[1:1] = [
+                "-W",
+                "ignore",
             ]
 
-    else:
-        if no_warnings:
-            cpython_cmd = [os.environ["PYTHON"], "-W", "ignore", filename]
+        if python_flag_m:
+            cpython_cmd += ["-m", os.path.basename(filename)]
+            os.chdir(os.path.dirname(filename))
         else:
-            cpython_cmd = [os.environ["PYTHON"], filename]
+            cpython_cmd.append(filename)
 
     if no_site:
         cpython_cmd.insert(1, "-S")
@@ -377,7 +406,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         # Would need to extract which "python" this is going to use.
         assert not coverage_mode, "Not implemented for binaries."
 
-        nuitka_call = [os.environ["NUITKA"]]
+        nuitka_call = [os.environ["PYTHON"], os.environ["NUITKA"]]
     else:
         if comparison_mode:
             nuitka_call = [
@@ -422,6 +451,9 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if noprefer_source:
         extra_options.append("--no-prefer-source")
 
+    if python_flag_m:
+        extra_options.append("--python-flag=-m")
+
     if coverage_mode:
         # Coverage modules hates Nuitka to re-execute, and so we must avoid
         # that.
@@ -441,9 +473,12 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if binary_python_path:
         addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
-    if keep_python_path or binary_python_path:
+    if (keep_python_path or binary_python_path) and not coverage_mode:
         extra_options.append("--execute-with-pythonpath")
 
+    if report:
+        extra_options.append("--report=%s" % report)
+
     if nofollow_imports:
         extra_options.append("--nofollow-imports")
 
@@ -458,10 +493,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         extra_options.append("--generate-c-only")
 
     for plugin_enabled in plugins_enabled:
-        extra_options.append("--plugin-enable=" + plugin_enabled)
+        extra_options.append("--enable-plugin=" + plugin_enabled)
 
     for plugin_disabled in plugins_disabled:
-        extra_options.append("--plugin-disable=" + plugin_disabled)
+        extra_options.append("--disable-plugin=" + plugin_disabled)
 
     for user_plugin in user_plugins:
         extra_options.append("--user-plugin=" + user_plugin)
@@ -472,6 +507,29 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if not noinclusion_log:
         extra_options.append("--show-modules-output=%s.inclusion.log" % filename)
 
+    if output_dir is not None:
+        extra_options.append("--output-dir=%s" % output_dir)
+    else:
+        # TODO: The run-tests uses NUITKA_EXTRA_OPTIONS still.
+        for extra_option in extra_options:
+            dir_match = re.search(r"--output-dir=(.*?)(\s|$)", extra_option)
+
+            if dir_match:
+                output_dir = dir_match.group(1)
+                break
+        else:
+            # The default.
+            output_dir = "."
+
+    for include_package in include_packages:
+        extra_options.append("--include-package=%s" % include_package)
+
+    for include_module in include_modules:
+        extra_options.append("--include-module=%s" % include_module)
+
+    # Progress bar is not used.
+    extra_options.append("--no-progressbar")
+
     # Now build the command to run Nuitka.
     if not two_step_execution:
         if module_mode:
@@ -499,24 +557,13 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if no_site:
             nuitka_cmd1.insert(len(nuitka_cmd1) - 1, "--python-flag=-S")
 
-    for extra_option in extra_options:
-        dir_match = re.search(r"--output-dir=(.*?)(\s|$)", extra_option)
-
-        if dir_match:
-            output_dir = dir_match.group(1)
-            break
-    else:
-        # The default.
-        output_dir = "."
-
     if module_mode:
-        nuitka_cmd2 = [
-            os.environ["PYTHON"],
-            "-W",
-            "ignore",
-            "-c",
-            "import %s" % os.path.basename(filename),
-        ]
+        module_name = os.path.basename(filename)
+
+        if module_name.endswith(".py"):
+            module_name = module_name[:-3]
+
+        nuitka_cmd2 = [os.environ["PYTHON"], "-W", "ignore", "-c", mini_script]
     else:
         exe_filename = os.path.basename(filename)
 
@@ -557,12 +604,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if output_dir:
             os.chdir(output_dir)
         else:
-            tmp_dir = tempfile.gettempdir()
-
-            # Try to avoid RAM disk /tmp and use the disk one instead.
-            if tmp_dir == "/tmp" and os.path.exists("/var/tmp"):
-                tmp_dir = "/var/tmp"
-
+            tmp_dir = getTempDir()
             os.chdir(tmp_dir)
 
         if trace_command:
@@ -600,13 +642,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
 
         for _i in range(5):
             with withPythonPathChange(nuitka_package_dir):
-                process = subprocess.Popen(
-                    args=nuitka_cmd1, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+                stdout_nuitka1, stderr_nuitka1, exit_nuitka1 = executeProcess(
+                    nuitka_cmd1
                 )
 
-            stdout_nuitka1, stderr_nuitka1 = process.communicate()
-            exit_nuitka1 = process.returncode
-
             if exit_nuitka1 != 0:
                 if (
                     not expect_failure
@@ -625,6 +664,8 @@ Stderr was:
 
                 exit_nuitka = exit_nuitka1
                 stdout_nuitka, stderr_nuitka = stdout_nuitka1, stderr_nuitka1
+                stdout_nuitka2 = b"not run due to compilation error:\n" + stdout_nuitka1
+                stderr_nuitka2 = stderr_nuitka1
             else:
                 # No execution second step for coverage mode.
                 if comparison_mode:
@@ -634,11 +675,14 @@ Stderr was:
                     if trace_command:
                         my_print("Nuitka command 2:", nuitka_cmd2)
 
+                    # Need full manual control, and not all Python versions allow using
+                    # context manager here, pylint: disable=consider-using-with
                     process = subprocess.Popen(
                         args=nuitka_cmd2, stdout=subprocess.PIPE, stderr=subprocess.PIPE
                     )
 
                     if send_kill:
+                        # Lambda is used immediately in same loop, pylint: disable=cell-var-from-loop
                         executeAfterTimePassed(
                             1.0,
                             lambda: killProcess("Nuitka compiled program", process.pid),
@@ -653,10 +697,7 @@ Stderr was:
                     if exit_nuitka in (-11, -6) and sys.platform != "nt":
                         nuitka_cmd2 = wrapCommandForDebuggerForSubprocess(*nuitka_cmd2)
 
-                        process = subprocess.Popen(
-                            args=nuitka_cmd2, stdin=subprocess.PIPE
-                        )
-                        process.communicate()
+                        callProcess(nuitka_cmd2, shell=False)
                 else:
                     exit_nuitka = exit_nuitka1
                     stdout_nuitka, stderr_nuitka = stdout_nuitka1, stderr_nuitka1
@@ -683,7 +724,11 @@ Stderr was:
 
         def makeComparisons(trace_result):
             exit_code_stdout = compareOutput(
-                "stdout", stdout_cpython, stdout_nuitka, ignore_warnings, syntax_errors
+                "stdout",
+                stdout_cpython,
+                stdout_nuitka2 if two_step_execution else stdout_nuitka,
+                ignore_warnings,
+                syntax_errors,
             )
 
             if ignore_stderr:
@@ -692,7 +737,7 @@ Stderr was:
                 exit_code_stderr = compareOutput(
                     "stderr",
                     stderr_cpython,
-                    stderr_nuitka,
+                    stderr_nuitka2 if two_step_execution else stderr_nuitka,
                     ignore_warnings,
                     syntax_errors,
                 )
@@ -760,9 +805,7 @@ Stderr was:
             nuitka_cmd.insert(len(nuitka_cmd) - 1, "--debugger")
 
             with withPythonPathChange(nuitka_package_dir):
-                process = subprocess.Popen(args=nuitka_cmd, stdin=subprocess.PIPE)
-
-            process.communicate()
+                callProcess(nuitka_cmd, shell=False)
 
         exit_code = exit_code_stdout or exit_code_stderr or exit_code_return
 
