@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,11 +26,10 @@ import re
 import sys
 
 from nuitka import Options, SourceCodeReferences
-from nuitka.__past__ import unicode  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import unicode
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version, python_version_str
-from nuitka.Tracing import general
-from nuitka.utils.FileOperations import putTextFileContents
+from nuitka.Tracing import general, my_print
 from nuitka.utils.Shebang import getShebangFromSource, parseShebang
 from nuitka.utils.Utils import getOS
 
@@ -116,7 +115,7 @@ def _detectEncoding2(source_file):
 def _readSourceCodeFromFilename2(source_filename):
     _installFutureFStrings()
 
-    # Detect the encoding.
+    # Detect the encoding, we do not know it, pylint: disable=unspecified-encoding
     with open(source_filename, "rU") as source_file:
         encoding = _detectEncoding2(source_file)
 
@@ -163,20 +162,31 @@ def readSourceCodeFromFilename(module_name, source_filename):
     else:
         source_code = _readSourceCodeFromFilename3(source_filename)
 
-    # Allow plug-ins to mess with source code. Test code calls this
+    # Allow plugins to mess with source code. Test code calls this
     # without a module and doesn't want changes from plugins.
     if module_name is not None:
         source_code_modified = Plugins.onModuleSourceCode(module_name, source_code)
     else:
         source_code_modified = source_code
 
-    if Options.shallPersistModifications() and source_code_modified != source_code:
-        orig_source_filename = source_filename + ".orig"
+    if Options.shallShowSourceModifications() and source_code_modified != source_code:
+        import difflib
 
-        if not os.path.exists(orig_source_filename):
-            putTextFileContents(filename=orig_source_filename, contents=source_code)
+        diff = difflib.unified_diff(
+            source_code.splitlines(),
+            source_code_modified.splitlines(),
+            "original",
+            "modified",
+            "",
+            "",
+            n=3,
+        )
 
-        putTextFileContents(filename=source_filename, contents=source_code_modified)
+        result = list(diff)
+
+        if result:
+            for line in result:
+                my_print(line, end="\n" if not line.startswith("---") else "")
 
     return source_code_modified
 
