@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,6 +24,9 @@ import os
 import sys
 
 from nuitka.PythonVersions import python_version
+from nuitka.Tracing import general
+
+from .Utils import withNoDeprecationWarning
 
 
 def _importFilePy3NewWay(filename):
@@ -76,6 +79,36 @@ def importFileAsModule(filename):
         return _importFilePy3NewWay(filename)
 
 
+_all_suffixes = None
+
+
+def getAllModuleSuffixes():
+    # Using global here, as this is for caching only
+    # pylint: disable=global-statement
+    global _all_suffixes
+
+    if _all_suffixes is None:
+        if python_version < 0x300:
+            import imp
+
+            _all_suffixes = []
+
+            for suffix, _mode, _module_type in imp.get_suffixes():
+                _all_suffixes.append(suffix)
+        else:
+            import importlib.machinery  # pylint: disable=I0021,import-error,no-name-in-module
+
+            _all_suffixes = (
+                importlib.machinery.EXTENSION_SUFFIXES
+                + importlib.machinery.SOURCE_SUFFIXES
+                + importlib.machinery.BYTECODE_SUFFIXES
+            )
+
+        _all_suffixes = tuple(_all_suffixes)
+
+    return _all_suffixes
+
+
 _shared_library_suffixes = None
 
 
@@ -116,11 +149,21 @@ def getSharedLibrarySuffix(preferred):
     return result
 
 
-def importFromFolder(module_name, path, must_exist):
+def _importFromFolder(logger, module_name, path, must_exist, message):
     """Import a module from a folder by adding it temporarily to sys.path"""
-    # May already be loaded
+
+    # Circular dependency here
+    from .FileOperations import isPathBelow
+
     if module_name in sys.modules:
-        return sys.modules[module_name]
+        # May already be loaded, but the wrong one from a ".pth" file of
+        # clcache that we then don't want to use.
+        if module_name != "clcache" or isPathBelow(
+            path=path, filename=sys.modules[module_name].__file__
+        ):
+            return sys.modules[module_name]
+        else:
+            del sys.modules[module_name]
 
     # Temporarily add the inline path of the module to the import path.
     sys.path.insert(0, path)
@@ -128,13 +171,19 @@ def importFromFolder(module_name, path, must_exist):
     # Handle case without inline copy too.
     try:
         return __import__(module_name)
-    except ImportError:
+    except (ImportError, SyntaxError) as e:
         if not must_exist:
             return None
 
-        sys.exit(
-            "Error, excepted inline copy of %r is in %r there." % (module_name, path)
+        exit_message = (
+            "Error, expected inline copy of '%s' to be in '%s', error was: %r."
+            % (module_name, path, e)
         )
+
+        if message is not None:
+            exit_message += "\n" + message
+
+        logger.sysexit(exit_message)
     finally:
         # Do not forget to remove it from sys.path again.
         del sys.path[0]
@@ -143,10 +192,53 @@ def importFromFolder(module_name, path, must_exist):
 def importFromInlineCopy(module_name, must_exist):
     """Import a module from the inline copy stage."""
 
-    return importFromFolder(
-        module_name=module_name,
-        path=os.path.join(
+    folder_name = os.path.normpath(
+        os.path.join(
             os.path.dirname(__file__), "..", "build", "inline_copy", module_name
-        ),
-        must_exist=must_exist,
+        )
     )
+
+    candidate_27 = folder_name + "_27"
+    candidate_35 = folder_name + "_35"
+
+    # Use specific versions if needed.
+    if python_version < 0x300 and os.path.exists(candidate_27):
+        folder_name = candidate_27
+    elif python_version < 0x360 and os.path.exists(candidate_35):
+        folder_name = candidate_35
+
+    return _importFromFolder(
+        module_name=module_name,
+        path=folder_name,
+        must_exist=must_exist,
+        message=None,
+        logger=general,
+    )
+
+
+_compile_time_modules = {}
+
+
+def importFromCompileTime(module_name, must_exist):
+    """Import a module from the compiled time stage.
+
+    This is not for using the inline copy, but the one from the actual
+    installation of the user. It suppresses warnings and caches the value
+    avoid making more __import__ calls that necessary.
+    """
+
+    if module_name not in _compile_time_modules:
+        with withNoDeprecationWarning():
+            try:
+                __import__(module_name)
+            except (ImportError, RuntimeError):
+                # Preventing a retry, converted to None for return
+                _compile_time_modules[module_name] = False
+            else:
+                _compile_time_modules[module_name] = sys.modules[module_name]
+
+    # Some code should only use this, after knowing it will be found. Complain if
+    # that is not the case.
+    assert _compile_time_modules[module_name] or not must_exist
+
+    return _compile_time_modules[module_name] or None
